// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef __WIN32__
#include <direct.h>
#else // NO WIN32
#include <dirent.h>
#endif // WIN32
#include <sys/types.h>
#include <sys/stat.h>
#include "globals.h"
#include "showloading.h"
#include "monstersprites.h"
#include "playersprite.h"
#include "extractmetabmf.h"
#include "errorhandler.h"

char basePath[1000];
char currentPath[1000];
char oldcurrentPath[1000];
char lastPath[1000];
bool startupComplete = false;


#ifdef __WIN32__
void configToWindowsPath() {
	char *ptr;
	while(ptr = strstr(lastPath, "/")) {
		*ptr = '\\';
	}
}
#endif // __WIN32__

void configInit() {
	oldcurrentPath[0] = '\0';
#ifdef __WIN32__
    char homepath[1000];
    if(!Win32_GetDocPath(homepath)) {
        DIE(ERROR_HOME, "Windows API");                                
    }
    printf("%s\r\n", homepath);
	sprintf(basePath, "%s\\BlinkenSisters\\", homepath);
#else
	sprintf(basePath, "%s/.lostpixels/", getenv("HOME"));
#endif // NO WIN32
	configResetAddOn();
	lastPath[0] = '\0';
	
	// Try to open VERSIOn file in home dir
	FILE* ifh = fopen(configGetPath("VERSION"), "r");
	if(ifh) {
		fclose(ifh);
	} else {
#ifndef __OGGPLAYER
		// We need to install our local copy of the basedata files
		sprintf(lastPath, "%s", currentPath);
#ifdef __WIN32__
		configToWindowsPath();
#endif		
		// We deal with a directory here - remove last delimeter
		lastPath[strlen(lastPath) - 1] = 0;
#ifdef __WIN32__
		_mkdir(lastPath);
#else // NO WIN32
		mkdir(lastPath, S_IRWXU);
#endif // WIN32

		// Write the VERSION file
		FILE* ofh = fopen(configGetPath("VERSION"), "w");
		if(!ofh) {
			DIE(ERROR_FILE_WRITE, configGetPath("VERSION"));
		}
		fprintf(ofh, "%s\n", LONGVERSION);
		fclose(ofh);
		
		// Now, we're finally ready to extract the basedata BMF
		sprintf(lastPath, "%sbasedata.bmf", RESPATH);
#ifdef __WIN32__
		configToWindowsPath();
#endif		
		extractMetaBMF(lastPath, true);
		
#ifdef CDEDITION
        configInstallAddOn("mz_escape.bmf");
        configInstallAddOn("mz_moviepack.bmf");
        configInstallAddOn("mz_testlevel.bmf");
        configInstallAddOn("mz_template.bmf");
        configInstallAddOn("mz_pnog.bmf");
        configInstallAddOn("jl_hell.bmf");
#endif // CDEDITION
		
#else // __OGGPLAYER
      DIE(ERROR_FILE_WRITE, "Please start BlinkenSisters/LostPixels first!");
#endif // __OGGPLAYER
	}
}

#ifdef CDEDITION
void configInstallAddOn(char* fname) {
	sprintf(lastPath, "%s%s", RESPATH, fname);
#ifdef __WIN32__
	configToWindowsPath();
#endif		
	extractMetaBMF(lastPath, true);
}     
#endif // CDEDITION

void configStartupComplete() {
	startupComplete = true;
}

void configSetBasePath(char *path) {
	sprintf(basePath, "%s", path);
	configResetAddOn();
}
	
char* configGetPath(char* fname) {
	sprintf(lastPath, "%s%s", currentPath, fname);
#ifdef __WIN32__
	configToWindowsPath();
#endif
	return lastPath;
}
	
void configSetAddOn(char* addOnName) {
	sprintf(currentPath, "%sADDON/%s/", basePath, addOnName);
	configReloadData();
}

void configResetAddOn() {
	sprintf(currentPath, "%s", basePath);
	configReloadData();
}

void configReloadData() {
	if(strcmp(currentPath, oldcurrentPath) == 0) {
		return;
	}
	sprintf(oldcurrentPath, "%s", currentPath);
#ifndef __OGGPLAYER
	if(startupComplete) {
		deInitShowLoading();
		initShowLoading("loading.jpg");
		showLoading();	
	}
#endif
	return;	
}
