// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "errorhandler.h"
#include "fonthandler.h"
#include <stdio.h>

#define PARSE_ERRORCODES
char* getErrorText(Uint32 errorcode) {
	static char intErrString[255];
	intErrString[0] = 0;
#define XX(errenum, errcode, errtxt) \
	if(errorcode == errcode) { \
		sprintf(intErrString, "%s", errtxt); \
	}
#include "errorcodes.h"
	return intErrString;
}
#undef PARSE_ERRORCODES

void dieWithError(Uint32 errorcode, const char* extrainfo, Uint32 linenum, const char* filename) {
	fprintf(stderr, "__          ___                           _ _ \n");
	fprintf(stderr, "\\ \\        / / |                         | | |\n");
	fprintf(stderr, " \\ \\  /\\  / /| |__   ___   ___  _ __  ___| | |\n");
	fprintf(stderr, "  \\ \\/  \\/ / | '_ \\ / _ \\ / _ \\| '_ \\/ __| | |\n");
	fprintf(stderr, "   \\  /\\  /  | | | | (_) | (_) | |_) \\__ \\_|_|\n");
	fprintf(stderr, "    \\/  \\/   |_| |_|\\___/ \\___/| .__/|___(_|_)\n");
	fprintf(stderr, "                               | |            \n");
	fprintf(stderr, "                               |_|            \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "The Blinkensisters team is proud to present you the following fatal error:\n");
	fprintf(stderr, "Error %d: %s\nError-Info: %s\nError at %s:%d\n",
						errorcode, getErrorText(errorcode), extrainfo, filename, linenum);
	fprintf(stderr, "\nPlease report this error to cavac@users.sourceforge.net!\n");
	exit(errorcode);
}

/* Display a full-screen INFO, WARNING or ERROR message for some time */
void displaymessage(int messagetype, char *message, Uint32 delay_ms) {
	SDL_Color message_INFO      = { 0X22, 0x8B, 0x22, 0 };
	SDL_Color message_WARNING   = { 0xFF, 0x7F, 0x00, 0 };
	SDL_Color message_ERROR     = { 0xEE, 0x00, 0x00, 0 };

	SDL_FillRect(gScreen, NULL, 0xffffffff); /* Fill screen with white color */

	switch (messagetype) {
		case displaymessage_INFO:
			renderFontHandlerText(10, 100, "Information", message_INFO, true, false, false);
			renderFontHandlerText(10, 200, message, message_INFO, true, false, true);
			break;
		case displaymessage_WARNING:
			renderFontHandlerText(10, 100, "Warning", message_WARNING, true, false, false);
			renderFontHandlerText(10, 200, message, message_WARNING, true, false, true);
			break;
		case displaymessage_ERROR:
			renderFontHandlerText(10, 100, "Information", message_ERROR, true, false, false);
			renderFontHandlerText(10, 200, message, message_ERROR, true, false, true);
			break;
	}

	SDL_Flip(gScreen);
	SDL_Delay(delay_ms);
}
