
// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "globals.h"
#include "errorhandler.h"
#include "fonthandler.h"
#include "SDL_ttf.h"
#ifdef SDLTTF_WORKAROUND
#include "showtext.h"
#include "menu.h"
#endif // SDLTTF_WORKAROUND


TTF_Font* menufont = 0;
TTF_Font* menufontsmall = 0;

#ifdef __OGGPLAYER
SDL_Color MENUCOLOR_ACTIVE   = { 0xff, 0xff, 0xff, 0 };
SDL_Color MENUCOLOR_INACTIVE = { 0xa0, 0xa0, 0xa0, 0 };
#endif // __OGGPLAYER


void initFontHandler() {
	printf("Init TTF\n");
	if(TTF_Init()) {
		DIE(ERROR_FONTLOAD, "TTF_Init");
	}

	menufont = TTF_OpenFont(configGetPath("Ghostmeat.ttf"), 50);
	if(!menufont) {
		DIE(ERROR_FONTLOAD, "Ghostmeat.ttf");
	}
	
	menufontsmall = TTF_OpenFont(configGetPath("Ghostmeat.ttf"), 20);
	if(!menufont) {
		DIE(ERROR_FONTLOAD, "Ghostmeat.ttf");
	}
}

void deInitFontHandler() {
	TTF_Quit();
}

void renderFontHandlerText(Sint32 x, Sint32 y, char* text, SDL_Color fontcolor, bool hcentered, bool vcentered, bool small) {
	SDL_Color fcol = { 0xff, 0xff, 0xff, 0 };

	SDL_Surface* temp;
	if(!TTF_WasInit()) {
		DIE(ERROR_FONTLOAD, "no TTF_Init");
	}
	if(!small) {
		temp = TTF_RenderText_Solid(menufont, text, fcol);
	} else {
		temp = TTF_RenderText_Solid(menufontsmall, text, fcol);
	}
		
	if(!temp) {
		printf("%s\n", TTF_GetError());
#ifndef SDLTTF_WORKAROUND
		DIE(ERROR_FONTLOAD, "Broken SDL_TTF, please downgrade");
#else // Workaround for broken SDL_TTF
        Uint32 twidth = getstringwidth(text);
        bool bright = false;
        if(fontcolor.r == MENUCOLOR_ACTIVE.r &&
            fontcolor.g == MENUCOLOR_ACTIVE.g &&
            fontcolor.b == MENUCOLOR_ACTIVE.b) {
            bright = true;
        }
        // Center text on screen if needed
    	if(hcentered) {
    		x = (SCR_WIDTH - twidth) / 2;
    	}
    	if(vcentered) {
    		y = (SCR_HEIGHT - 16) / 2;
    	}
    	drawstring(x, y, text, gScreen, bright);
    	return;
#endif // End of workaround
	}
	
	SDL_Surface* tmp = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);

	
	// Center text on screen if needed
	if(hcentered) {
		x = (SCR_WIDTH - tmp->w) / 2;
	}
	if(vcentered) {
		y = (SCR_HEIGHT - tmp->h) / 2;
	}

	// Calculate offset in tiles-pixmap
	Uint32 xorigoffs = 0;
	Uint32 yorigoffs = 0;
	if(x < 0) {
		xorigoffs = -x;
	}
	if(y < 0) {
		yorigoffs = -y;
	}
	
	// Calculate HEIGHT/WIDTH of text to copy
	Uint32 width = tmp->w;
	Uint32 height = tmp->h;
	if(x > (SCR_WIDTH - tmp->w)) {
		width = SCR_WIDTH - x;
	}
	if(y > (SCR_HEIGHT - tmp->h)) {
		height = SCR_HEIGHT - y;
	}
	
	// -- Let's get blitting!!!
	if (SDL_MUSTLOCK(tmp))
		if (SDL_LockSurface(tmp) < 0)
			return;	

	Uint32 scrpitch = gScreen->pitch / 4;
	Uint32 tilpitch = tmp->pitch / 4;
	
	Uint32 color;
	for(Uint32 ybg = yorigoffs; ybg < height; ybg++) { 
		for(Uint32 xbg = xorigoffs; xbg < width; xbg++) {
			color = ((Uint32*)tmp->pixels)[xbg + ybg * tilpitch];
			
			if(color != 0x00000000) {
				((Uint32*)gScreen->pixels)[xbg + x + (y + ybg) * scrpitch] = SDL_MapRGB(gScreen->format, fontcolor.r, fontcolor.g, fontcolor.b);
			}
			
		}
	}
	
	// Unlock if needed
	if (SDL_MUSTLOCK(tmp))
		SDL_UnlockSurface(tmp);


	
	SDL_FreeSurface(tmp);
}

