// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See LICENSE for licensing information
//


#include "globals.h"
#include "intro.h"
#include "sound.h"
#include "showtext.h"
#include "drawprimitives.h"
#include "debug.h"
#include "background.h"
#include "tiles.h"
#include "errorhandler.h"
#include "engine.h"
#include "joystick.h"

SDL_Surface *introFG;

Uint32 bgoffs = 0;
Uint32 tmpbgoffs;
Uint32 fgtransp;
Uint32 fgMinX, fgMaxX, fgMinY, fgMaxY;

bool blinkOn = true;
int bgMusicSelect = 0;


void initIntro()
{
	SDL_Surface *temp = IMG_Load(configGetPath("intro.png"));
	if(!temp) {
		DIE(ERROR_IMAGE_READ, "intro.png");
	}
	introFG = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
		
	// Looking for the transparent color in Intro-Foreground
	// For simplicity, take the uppermost left pixel (e.g. the first pixels)
	// as an indicator for the transparent one
	
	// Lock surface if needed
	if (SDL_MUSTLOCK(introFG))
		if (SDL_LockSurface(introFG) < 0) 
			return;
	fgtransp = ((Uint32*)introFG->pixels)[0];
	
	// Check for the empty borders of the foreground so we can ignore them on rendering
	fgMinX = SCR_WIDTH;
	fgMinY = SCR_HEIGHT;
	fgMaxX = 0;
	fgMaxY = 0;
	Uint32 i;
	Uint32 j;
	Uint32 pitchFG = introFG->pitch / 4;
	
	for(i=0; i < SCR_WIDTH; i++) {
		for(j=0; j < SCR_HEIGHT; j++) {
			if(((Uint32 *)introFG->pixels)[j * pitchFG + i] != fgtransp) {
				if(i < fgMinX) {
					fgMinX = i;
				}
				if(j < fgMinY) {
					fgMinY = j;
				}
				if(i > fgMaxX) {
					fgMaxX = i;
				}
				if(j > fgMaxY) {
					fgMaxY = j;
				}
			}
		}
	}
	
	
	// Unlock if needed
	if (SDL_MUSTLOCK(introFG)) 
		SDL_UnlockSurface(introFG);
	
	gLastTick = SDL_GetTicks();
	
	initBackground("introbg.jpg");
	initTiles("bathtiles.bmp");
	
	switch(bgMusicSelect) {
		case 0:
			soundStartMusic("introMusic.ogg", true);
		break;
		case 1:
			soundStartMusic("bs_ingame2.ogg", true);
		break;
		case 2:
			soundStartMusic("bs_ingame0.ogg", true);
		break;
		case 3:
			soundStartMusic("bs_bonuslevel0.ogg", true);
		break;
			
	}
	bgMusicSelect++;
	if(bgMusicSelect == 4) {
		bgMusicSelect = 0;
	}
		
}

void deInitIntro ()
{
	soundStopMusic();
	SDL_FreeSurface(introFG);
	deInitBackground();
	deInitTiles();
}


void renderIntro()
{
	
	// Ask SDL for the time in milliseconds
	Uint32 tick = SDL_GetTicks();
	
	// Delay a bit if we're too fast
	while (tick <= gLastTick) {
		SDL_Delay(1);
		//sprintf(errorMessage, "TOO FAST!");
		tick = SDL_GetTicks();
	}

	// Physics Engine
	while (gLastTick < tick)
	{
		gLastTick += 1000 / PHYSICSFPS;
		bgoffs++;
		if(bgoffs >= INTRO_BGWIDTH) {
			bgoffs -= INTRO_BGWIDTH;
		}
	}
	
	
	Uint32 i;
	Uint32 j;
	Uint32 pitch = gScreen->pitch / 4;
	Uint32 pitchFG = introFG->pitch / 4;

	// This intro is quite simply made, so the foreground bitmap must be EXACTLY the same
	// size as our screen!
	
	// Lock screen if needed
	if (SDL_MUSTLOCK(gScreen))
		if (SDL_LockSurface(gScreen) < 0) 
			return;

	// Background
	drawBackground(bgoffs, 0);	
	
	// Foreground
	// Lock foreground if needed
	if (SDL_MUSTLOCK(introFG))
		if (SDL_LockSurface(introFG) < 0) 
			return;

	// Draw non-transparent parts of FG
	for(i=fgMinX; i < fgMaxX; i++) {
		for(j=fgMinY; j < fgMaxY; j++) {
			if(((Uint32 *)introFG->pixels)[j * pitchFG + i] != fgtransp) {
				((Uint32 *)gScreen->pixels)[j * pitch + i] = 
						((Uint32 *)introFG->pixels)[j * pitchFG + i];
			}
		}
	}
	
	 
	
	// Unlock foreground if needed
	if (SDL_MUSTLOCK(introFG)) 
		SDL_UnlockSurface(introFG);
	

	// Draw Version String and something blinking
	
#ifndef CAVAC_RELEASEMODE
	drawrect(40,455,300,20,0x00000000);
#endif //  CAVAC_RELEASEMODE

#ifndef BLINKOMATEDITION
	drawstring(45, 460, LONGVERSION, gScreen);
#else
	drawstring(45, 460, "Blink-O-Mat " LONGVERSION, gScreen);
#endif // BLINKOMATEDITION
	
	drawstring(150, 370, "Press Space to start", gScreen);
	
#ifndef CAVAC_RELEASEMODE
	Uint32 tmpcol;
	if(blinkOn) {
		tmpcol = 0x00FFFFFF;
	} else {
		tmpcol = 0x00000000;
	}
	blinkOn = !blinkOn;

	drawcircle(20,460,15,tmpcol);


#ifdef SHOWFRAMERATE
	displayFPSCounter();
#endif //
#endif // CAVAC_RELEASEMODE

	// Unlock screen if needed
	if (SDL_MUSTLOCK(gScreen)) 
		SDL_UnlockSurface(gScreen);
	
	// Tell SDL to update the whole screen
	SDL_Flip(gScreen); 
	
}



void displayIntro()
{
	
	initIntro();
	
	bool displayIntro = true;
	while (displayIntro)
	{
		// Render stuff
		renderIntro();
		
		// Stop intro if sound has finished playing
		if(playOnceFinished) {
			displayIntro = false;
		}
		
		// Poll for events, and handle the ones we care about.
		Uint32 joymove = getJoystickMoves();
		if(joymove & JOYSTICK_JUMP || joymove & JOYSTICK_ACTION) {
			displayIntro = false;
			attracktModeRunning = false;
		}
		SDL_Event event;
		while (SDL_PollEvent(&event) && displayIntro) 
		{
			switch (event.type) 
			{
				case SDL_KEYDOWN:
					break;
				case SDL_KEYUP:
					// If escape is pressed, return (and thus, quit)
					if (event.key.keysym.sym != SDLK_ESCAPE) {
						displayIntro = false;
						attracktModeRunning = false;

#ifdef ALLOW_BOSSKEY
					} else if (event.key.keysym.sym == SDLK_ESCAPE) {
						exit(0);
#endif // ALLOW_BOSSKEY
					}
					break;
				case SDL_QUIT:
					exit(0);
			}
		}
	}
	
	deInitIntro();
}

