// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "globals.h"
#include "menu.h"
#include "fonthandler.h"
#include "errorhandler.h"
#include "memops.h"
#include "sound.h"
#include "joystick.h"
#include "showvideo.h"
#include <string.h>
#include "httpclient.h"
#include "drawprimitives.h"
#include "blending.h"
#include "extractmetabmf.h"
#include "gameengine.h"
#include "intro.h"
#include "engine.h"

SDL_Surface* menubg = 0;
SDL_Surface* menuonlinebg = 0;
SDL_Surface* menuneedhelpbg = 0;

bool menuIsVideoTest = false;
Uint32 attrackModeType = 0;

SDL_Color MENUCOLOR_ACTIVE   = { 0xff, 0xff, 0xff, 0 };
SDL_Color MENUCOLOR_INACTIVE = { 0xa0, 0xa0, 0xa0, 0 };

#ifndef DISABLE_NETWORK
extern char * proxyurl ; 
#endif // DISABLE_NETWORK

void initMenu() {
	SDL_Surface* temp = IMG_Load(configGetPath("menubg.jpg"));
	if(!temp) {
		DIE(ERROR_IMAGE_READ, "menubg.jpg");
	}
	menubg = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	
	temp = IMG_Load(configGetPath("menuonlinebg.jpg"));
	if(!temp) {
		DIE(ERROR_IMAGE_READ, "menuonlinebg.jpg");
	}
	menuonlinebg = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	
	temp = IMG_Load(configGetPath("menuneedhelpbg.jpg"));
	if(!temp) {
		DIE(ERROR_IMAGE_READ, "menuneedhelpbg.jpg");
	}
	menuneedhelpbg = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	
}

void deInitMenu() {
	SDL_FreeSurface(menubg);
	SDL_FreeSurface(menuonlinebg);
	SDL_FreeSurface(menuneedhelpbg);
	menubg = 0;
}

bool menuDisplay() {
#ifdef BLINKOMATEDITION
//	menuAttrackMode();
//	return true;
#endif // BLINKOMATEDITION
	
	char* menutexte[] = {"Play",
#ifndef DISABLE_NETWORK
                                    "Download",
#endif // DISABLE_NETWORK
                                               "Contribute","Quit", NULL};
	
	Uint32 sel = 1;
	Uint32 max = sizeof(menutexte)/sizeof(menutexte[0])-1;
	if(menuIsVideoTest) {
		max = 6;
	}
	bool menuRunning = true;
	bool joystickIdle = false;
	
	soundStartMusic("ADDON/LostPixels/menuMusic.ogg", true);
	while(menuRunning) {
		
		// Lock Surface if needed
		if (SDL_MUSTLOCK(gScreen))
			if (SDL_LockSurface(gScreen) < 0) 
				return false;	
		
		if (SDL_MUSTLOCK(menubg))
			if (SDL_LockSurface(menubg) < 0) 
				return false;			
		
		Uint32 bigpitch = gScreen->pitch;
		Uint32 bigpitchBG = menubg->pitch;
		
		long fc_src_long = (long)menubg->pixels;
		long fc_dst_long = (long)gScreen->pixels;
		
		for(Uint32 j=0; j < SCR_HEIGHT; j++) {
			fc_src = (fastcopy *)fc_src_long;
			fc_dest = (fastcopy *)fc_dst_long;
			*fc_dest = *fc_src;
			fc_dst_long += bigpitch;
			fc_src_long += bigpitchBG;
		}
		
		Uint32 cnt=0;
		while(menutexte[cnt] != NULL) {
			if ((cnt+1)==sel) {
				char s[255];
				sprintf(s,"* %s *",menutexte[cnt]);
				renderFontHandlerText(10, 100+cnt*70, s, MENUCOLOR_ACTIVE, true, false, false); /* selected */
			} else {
				renderFontHandlerText(10, 100+cnt*70, menutexte[cnt], MENUCOLOR_INACTIVE, true, false, false); /* not selected */
			}
			cnt++ ;
		}
		
		
		// Unlock Surface if needed
		if (SDL_MUSTLOCK(menubg)) 
			SDL_UnlockSurface(menubg);
		if (SDL_MUSTLOCK(gScreen)) 
			SDL_UnlockSurface(gScreen);
		SDL_Flip(gScreen); /* Update whole screen */
		
		// Handle Joystick
		Uint32 joymove = getJoystickMoves();
		if(joymove == JOYSTICK_NONE) {
			joystickIdle = true;
		}
		if(joystickIdle && joymove != JOYSTICK_NONE) {
			if((joymove & JOYSTICK_UP)) {
				soundPlayFX(FX_MENU);
				sel--;
				if(!sel) {
					sel = max;
				}
			} else if((joymove & JOYSTICK_DOWN)) {
				soundPlayFX(FX_MENU);
				sel++;
				if(sel > max) {
					sel = 1;
				}
			} else if((joymove & JOYSTICK_ACTION)) {
				// FIXME: Handle CONFIGURE option
				soundPlayFX(FX_MENU);
				if(strcmp(menutexte[sel-1],"Contribute")==0) {
					menuShowNeedHelp();
#ifndef DISABLE_NETWORK
				} else if(strcmp(menutexte[sel-1],"Download")==0) {
					menuOnlineDisplay();
#endif // DISABLE_NETWORK
				} else if(strcmp(menutexte[sel-1],"Play")==0) {
					menuAddonDisplay();
				} else {
					menuRunning = false;
				}			} 
			joystickIdle = false;
		}
		
		
		// Handle Keyboard
		SDL_Event event;
		while (SDL_PollEvent(&event)) 
		{
			switch (event.type) 
			{
				case SDL_KEYUP:
					if ((event.key.keysym.sym == SDLK_UP) ||(event.key.keysym.sym == SDLK_w)) {
						soundPlayFX(FX_MENU);
						sel--;
						if(!sel) {
							sel = max;
						}
					} else if ((event.key.keysym.sym == SDLK_DOWN) ||(event.key.keysym.sym == SDLK_s)) {
						soundPlayFX(FX_MENU);
						sel++;
						if(sel > max) {
							sel = 1;
						}
					} else if (event.key.keysym.sym == SDLK_RETURN) {
						// FIXME: Handle CONFIGURE option
						soundPlayFX(FX_MENU);
						if(strcmp(menutexte[sel-1],"Contribute")==0) {
							menuShowNeedHelp();
#ifndef DISABLE_NETWORK
						} else if(strcmp(menutexte[sel-1],"Download")==0) {
							menuOnlineDisplay();
#endif // DISABLE_NETWORK
						} else if(strcmp(menutexte[sel-1],"Play")==0) {
							menuAddonDisplay();
						} else {
							menuRunning = false;
						}
#ifdef ALLOW_BOSSKEY
					} else if (event.key.keysym.sym == SDLK_ESCAPE) {
						exit(0);
#endif // ALLOW_BOSSKEY
					}
					
					break;
			}
		}
		
		// Check for "Attrack mode"
		if(playOnceFinished) {
			menuAttrackMode();
		}
		
		SDL_Delay(50);
	}
	
	if(!menuIsVideoTest) {
		soundStopMusic();
	}

	return false;
}

#ifndef DISABLE_NETWORK

// -------- ONLINE DISPLAY ---------------------
struct WEBTOC {
	char name[100];
	char desc[100];
	char size[10];
	char fname[100];
};

bool menuOnlineDisplay() {
	
	SDL_Color MENUCOLOR_ACTIVE   = { 0xaf, 0xff, 0xff, 0 };
	//SDL_Color MENUCOLOR_INACTIVE = { 0x40, 0xa0, 0xa0, 0 };
	SDL_Color MENUCOLOR_INACTIVE = { 0x70, 0x70, 0x70, 0 };
	
	// -- Prepare http client and download TOC -> display a progress bar --
	char httpfile[1000];
	WEBTOC files[100];
	
	// Lock Surface if needed
	if (SDL_MUSTLOCK(gScreen))
		if (SDL_LockSurface(gScreen) < 0) 
			return false;	
	
	if (SDL_MUSTLOCK(menuonlinebg))
		if (SDL_LockSurface(menuonlinebg) < 0) 
			return false;	
	
	
	Uint32 bigpitch = gScreen->pitch;
	Uint32 bigpitchBG = menuonlinebg->pitch;
	
	long fc_src_long = (long)menuonlinebg->pixels;
	long fc_dst_long = (long)gScreen->pixels;
	
	for(Uint32 j=0; j < SCR_HEIGHT; j++) {
		fc_src = (fastcopy *)fc_src_long;
		fc_dest = (fastcopy *)fc_dst_long;
		*fc_dest = *fc_src;
		fc_dst_long += bigpitch;
		fc_src_long += bigpitchBG;
	}
	
	renderFontHandlerText(10, 100, "Please wait...", MENUCOLOR_ACTIVE, true, false, false);
	// Unlock Surface if needed
	if (SDL_MUSTLOCK(menuonlinebg)) 
		SDL_UnlockSurface(menuonlinebg);
	if (SDL_MUSTLOCK(gScreen)) 
		SDL_UnlockSurface(gScreen);
	SDL_Flip(gScreen); /* Update whole screen */
	
	
	sprintf(httpfile, "%s%s", HTTPBASEDIR, HTTPTOCFILE);
	HTTPClient web(HTTPSERVER, HTTPPORT, HTTPDEBUG, proxyurl);
	if(!web.get(httpfile)) {
		DIE(ERROR_HTTPGET, httpfile);
	}
	if(!web.save(configGetPath("temp_webtoc"))) {
		DIE(ERROR_FILE_WRITE, configGetPath("temp_webtoc"));
	}
	
	
	Uint32 sel = 1;
	Uint32 offs = 1;
	Uint32 max = 0; /* TODO: compute max as the size of the array menutexte */
	FILE *toc = fopen(configGetPath("temp_webtoc"), "r");
	if(!toc) {
		DIE(ERROR_FILE_READ, configGetPath("temp_webtoc"));
	}
	char line[1001];
	char* tmpchar;
	char* tmpchar2;
	while(!feof(toc)) {
		max++;
		if(!fgets(line, 1000, toc)) {
			break;
		}
		while((tmpchar = strchr(line, '\n'))) {
			*tmpchar = 0;
		}
		while((tmpchar = strchr(line, '\r'))) {
			*tmpchar = 0;
		}
		
		
		if(!(tmpchar = strchr(line, '|'))) {
			DIE(ERROR_HTTPTOC, "1|2");
		}
		*tmpchar = 0;
		sprintf(files[max].name, "%s", line);
		tmpchar++;
		tmpchar2 = tmpchar;
		
		if(!(tmpchar = strchr(tmpchar, '|'))) {
			DIE(ERROR_HTTPTOC, "2|3");
		}
		*tmpchar = 0;
		sprintf(files[max].desc, "%s", tmpchar2);
		tmpchar++;
		tmpchar2 = tmpchar;
		
		if(!(tmpchar = strchr(tmpchar, '|'))) {
			DIE(ERROR_HTTPTOC, "3|4");
		}
		*tmpchar = 0;
		sprintf(files[max].size, "%s", tmpchar2);
		tmpchar++;
		tmpchar2 = tmpchar;
		sprintf(files[max].fname, "%s", tmpchar2);
	}
	fclose(toc);
	
	bool menuRunning = true;
	bool joystickIdle = false;
	while(menuRunning) {
		
		// Lock Surface if needed
		if (SDL_MUSTLOCK(gScreen))
			if (SDL_LockSurface(gScreen) < 0) 
				return false;	
		
		if (SDL_MUSTLOCK(menuonlinebg))
			if (SDL_LockSurface(menuonlinebg) < 0) 
				return false;	
		
		
		Uint32 bigpitch = gScreen->pitch;
		Uint32 bigpitchBG = menuonlinebg->pitch;
		
		long fc_src_long = (long)menuonlinebg->pixels;
		long fc_dst_long = (long)gScreen->pixels;
		
		for(Uint32 j=0; j < SCR_HEIGHT; j++) {
			fc_src = (fastcopy *)fc_src_long;
			fc_dest = (fastcopy *)fc_dst_long;
			*fc_dest = *fc_src;
			fc_dst_long += bigpitch;
			fc_src_long += bigpitchBG;
		}
		
		// Scroll to the correct position
		while( ((Sint32)sel - (Sint32)offs) > 3) {
			offs++;
		}
		while( ((Sint32)sel - (Sint32)offs) < 0) {
			offs--;
		}
		
		Uint32 tmpmax = max;
		if(tmpmax > (offs + 3)) {
			tmpmax = offs + 3;
		}
		
		for(Uint32 i= offs; i <= tmpmax; i++) {
			int my_y = 70+(i - offs) *90;
			
			if(i < max) {
				blend_darkenRect(20, my_y, 600, 80, 0x00a0a0a0);
				if(i == sel) {
					drawrect(20, my_y, 600, 1, 0xd0d0d0);
					drawrect(20, my_y, 1, 80, 0xd0d0d0);
					drawrect(20, my_y+79, 600, 1, 0xd0d0d0);
					drawrect(619, my_y, 1, 80, 0xd0d0d0);
				} else {
					drawrect(20, my_y, 600, 1, 0x606060);
					drawrect(20, my_y, 1, 80, 0x606060);
					drawrect(20, my_y+79, 600, 1, 0x606060);
					drawrect(619, my_y, 1, 80, 0x606060);
				}
			}
			
			if(i == max) {
				if(i == sel) {
					renderFontHandlerText(10, my_y + 10, "Back", MENUCOLOR_ACTIVE, true, false, false);
				} else {
					renderFontHandlerText(10, my_y + 10, "Back", MENUCOLOR_INACTIVE, true, false, false);
				}
			} else {
				if(i == sel) {
					renderFontHandlerText(23, my_y + 3, files[i].name, MENUCOLOR_ACTIVE, false, false, false);
					renderFontHandlerText(23, my_y + 57, files[i].desc, MENUCOLOR_ACTIVE, false, false, true);
					renderFontHandlerText(520, my_y + 57, files[i].size, MENUCOLOR_ACTIVE, false, false, true);
				} else {
					renderFontHandlerText(23, my_y + 3, files[i].name, MENUCOLOR_INACTIVE, false, false, false);
					renderFontHandlerText(23, my_y + 57, files[i].desc, MENUCOLOR_INACTIVE, false, false, true);
					renderFontHandlerText(520, my_y + 57, files[i].size, MENUCOLOR_INACTIVE, false, false, true);
				}					
			}
			
			
		}
		
		// Unlock Surface if needed
		if (SDL_MUSTLOCK(menuonlinebg)) 
			SDL_UnlockSurface(menuonlinebg);
		if (SDL_MUSTLOCK(gScreen)) 
			SDL_UnlockSurface(gScreen);
		SDL_Flip(gScreen); /* Update whole screen */
		
		// Handle Joystick
		Uint32 joymove = getJoystickMoves();
		if(joymove == JOYSTICK_NONE) {
			joystickIdle = true;
		}
		if(joystickIdle && joymove != JOYSTICK_NONE) {
			if((joymove & JOYSTICK_UP)) {
				soundPlayFX(FX_MENU);
				sel--;
				if(!sel) {
					sel = max;
				}
			} else if((joymove & JOYSTICK_DOWN)) {
				soundPlayFX(FX_MENU);
				sel++;
				if(sel > max) {
					sel = 1;
				}
			} else if((joymove & JOYSTICK_ACTION)) {
				// FIXME: Handle CONFIGURE option
				soundPlayFX(FX_MENU);
				soundPlayFX(FX_MENU);
				if(sel != max) {
					sprintf(httpfile, "%s%s", HTTPBASEDIR, files[sel].fname);
					if(!web.get(httpfile)) {
						DIE(ERROR_HTTPGET, httpfile);
					}
					if(!web.save(configGetPath("temp_web.bmf"))) {
						DIE(ERROR_FILE_WRITE, configGetPath("temp_web.bmf"));
					}
					if(!extractMetaBMF(configGetPath("temp_web.bmf"))) {
						DIE(ERROR_BMFPARSE, configGetPath("temp_web.bmf"));
					}
					soundPlayFX(FX_MENU);
				} else {
					menuRunning = false;
				}
			} 
			joystickIdle = false;
		}
		
		
		// Handle Keyboard
		SDL_Event event;
		while (SDL_PollEvent(&event)) 
		{
			switch (event.type) 
			{
				case SDL_KEYUP:
					if ((event.key.keysym.sym == SDLK_UP) || (event.key.keysym.sym == SDLK_w)) {
						soundPlayFX(FX_MENU);
						sel--;
						if(!sel) {
							sel = max;
						}
					} else if ((event.key.keysym.sym == SDLK_DOWN) || (event.key.keysym.sym == SDLK_s)) {
						soundPlayFX(FX_MENU);
						sel++;
						if(sel > max) {
							sel = 1;
						}
					} else if (event.key.keysym.sym == SDLK_RETURN) {
						// FIXME: Handle CONFIGURE option
						soundPlayFX(FX_MENU);
						if(sel != max) {
							sprintf(httpfile, "%s%s", HTTPBASEDIR, files[sel].fname);
							if(!web.get(httpfile)) {
								DIE(ERROR_HTTPGET, httpfile);
							}
							if(!web.save(configGetPath("temp_web.bmf"))) {
								DIE(ERROR_FILE_WRITE, configGetPath("temp_web.bmf"));
							}
							if(!extractMetaBMF(configGetPath("temp_web.bmf"))) {
								DIE(ERROR_BMFPARSE, configGetPath("temp_web.bmf"));
							}
							soundPlayFX(FX_MENU);
						} else {
							menuRunning = false;
						}
#ifdef ALLOW_BOSSKEY
					} else if (event.key.keysym.sym == SDLK_ESCAPE) {
						exit(0);
#endif // ALLOW_BOSSKEY
					}
					
					break;
			}
		}
		
		SDL_Delay(50);
	}
	
	return true;
}
#endif // DISABLE_NETWORK


// -------- ADDON DISPLAY ---------------------
struct ADDONTOC {
	char name[100];
	char desc[100];
	char fname[100];
};

bool menuAddonDisplay() {
	
	ADDONTOC files[100];
	
	Uint32 sel = 1;
	Uint32 offs = 1;
	Uint32 max = 0; /* TODO: compute max as the size of the array menutexte */
	FILE *toc = fopen(configGetPath("addons.dat"), "r");
	if(!toc) {
		//DIE(ERROR_FILE_READ, configGetPath("addons.dat"));
		max = 1;
	} else {
		char line[1001];
		char* tmpchar;
		char* tmpchar2;
		while(!feof(toc)) {
			max++;
			if(!fgets(line, 1000, toc)) {
				break;
			}
			while((tmpchar = strchr(line, '\n'))) {
				*tmpchar = 0;
			}
			while((tmpchar = strchr(line, '\r'))) {
				*tmpchar = 0;
			}
			
			
			if(!(tmpchar = strchr(line, '|'))) {
				DIE(ERROR_HTTPTOC, "1|2");
			}
			*tmpchar = 0;
			sprintf(files[max].name, "%s", line);
			tmpchar++;
			tmpchar2 = tmpchar;
			
			if(!(tmpchar = strchr(tmpchar, '|'))) {
				DIE(ERROR_HTTPTOC, "2|3");
			}
			*tmpchar = 0;
			sprintf(files[max].desc, "%s", tmpchar2);
			tmpchar++;
			tmpchar2 = tmpchar;
			
			sprintf(files[max].fname, "%s", tmpchar2);
		}
		fclose(toc);
	}
	
	bool menuRunning = true;
	bool joystickIdle = false;
	while(menuRunning) {
		
		// Lock Surface if needed
		if (SDL_MUSTLOCK(gScreen))
			if (SDL_LockSurface(gScreen) < 0) 
				return false;	
		
		if (SDL_MUSTLOCK(menuonlinebg))
			if (SDL_LockSurface(menuonlinebg) < 0) 
				return false;	
		
		
		Uint32 bigpitch = gScreen->pitch;
		Uint32 bigpitchBG = menuonlinebg->pitch;
		
		long fc_src_long = (long)menuonlinebg->pixels;
		long fc_dst_long = (long)gScreen->pixels;
		
		for(Uint32 j=0; j < SCR_HEIGHT; j++) {
			fc_src = (fastcopy *)fc_src_long;
			fc_dest = (fastcopy *)fc_dst_long;
			*fc_dest = *fc_src;
			fc_dst_long += bigpitch;
			fc_src_long += bigpitchBG;
		}
		
		// Scroll to the correct position
		while( ((Sint32)sel - (Sint32)offs) > 3) {
			offs++;
		}
		while( ((Sint32)sel - (Sint32)offs) < 0) {
			offs--;
		}
		
		Uint32 tmpmax = max;
		if(tmpmax > (offs + 3)) {
			tmpmax = offs + 3;
		}
		
		for(Uint32 i= offs; i <= tmpmax; i++) {
			int my_y = 70+(i - offs) *90;
			
			if(i < max) {
				blend_darkenRect(20, my_y, 600, 80, 0x00a0a0a0);
				if(i == sel) {
					drawrect(20, my_y, 600, 1, 0xd0d0d0);
					drawrect(20, my_y, 1, 80, 0xd0d0d0);
					drawrect(20, my_y+79, 600, 1, 0xd0d0d0);
					drawrect(619, my_y, 1, 80, 0xd0d0d0);
				} else {
					drawrect(20, my_y, 600, 1, 0x606060);
					drawrect(20, my_y, 1, 80, 0x606060);
					drawrect(20, my_y+79, 600, 1, 0x606060);
					drawrect(619, my_y, 1, 80, 0x606060);
				}
			}
			
			if(i == max) {
				if(i == sel) {
					renderFontHandlerText(10, my_y + 10, "Back", MENUCOLOR_ACTIVE, true, false, false);
				} else {
					renderFontHandlerText(10, my_y + 10, "Back", MENUCOLOR_INACTIVE, true, false, false);
				}
			} else {
				if(i == sel) {
					renderFontHandlerText(23, my_y + 3, files[i].name, MENUCOLOR_ACTIVE, false, false, false);
					renderFontHandlerText(23, my_y + 57, files[i].desc, MENUCOLOR_ACTIVE, false, false, true);
				} else {
					renderFontHandlerText(23, my_y + 3, files[i].name, MENUCOLOR_INACTIVE, false, false, false);
					renderFontHandlerText(23, my_y + 57, files[i].desc, MENUCOLOR_INACTIVE, false, false, true);
				}					
			}
			
			
		}
		
		// Unlock Surface if needed
		if (SDL_MUSTLOCK(menuonlinebg)) 
			SDL_UnlockSurface(menuonlinebg);
		if (SDL_MUSTLOCK(gScreen)) 
			SDL_UnlockSurface(gScreen);
		SDL_Flip(gScreen); /* Update whole screen */
		
		// Handle Joystick
		Uint32 joymove = getJoystickMoves();
		if(joymove == JOYSTICK_NONE) {
			joystickIdle = true;
		}
		if(joystickIdle && joymove != JOYSTICK_NONE) {
			if((joymove & JOYSTICK_UP)) {
				soundPlayFX(FX_MENU);
				sel--;
				if(!sel) {
					sel = max;
				}
			} else if((joymove & JOYSTICK_DOWN)) {
				soundPlayFX(FX_MENU);
				sel++;
				if(sel > max) {
					sel = 1;
				}
			} else if((joymove & JOYSTICK_ACTION)) {
				// FIXME: Handle CONFIGURE option
				soundPlayFX(FX_MENU);
				if(sel != max) {
					soundStopMusic();
					configSetAddOn(files[sel].fname);							
					initGame();
					playGame();
					configResetAddOn();
					soundStartMusic("ADDON/LostPixels/menuMusic.ogg");
				} else {
					menuRunning = false;
				}
			} 
			joystickIdle = false;
		}
		
		
		// Handle Keyboard
		SDL_Event event;
		while (SDL_PollEvent(&event)) 
		{
			switch (event.type) 
			{
				case SDL_KEYUP:
					if ((event.key.keysym.sym == SDLK_UP) || (event.key.keysym.sym == SDLK_w)) {
						soundPlayFX(FX_MENU);
						sel--;
						if(!sel) {
							sel = max;
						}
					} else if ((event.key.keysym.sym == SDLK_DOWN) ||(event.key.keysym.sym == SDLK_s)) {
						soundPlayFX(FX_MENU);
						sel++;
						if(sel > max) {
							sel = 1;
						}
					} else if (event.key.keysym.sym == SDLK_RETURN) {
						soundPlayFX(FX_MENU);
						if(sel != max) {
							soundStopMusic();
							configSetAddOn(files[sel].fname);							
							initGame();
							playGame();
							configResetAddOn();
							soundStartMusic("ADDON/LostPixels/menuMusic.ogg");
						} else {
							menuRunning = false;
						}
#ifdef ALLOW_BOSSKEY
					} else if (event.key.keysym.sym == SDLK_ESCAPE) {
						exit(0);
#endif // ALLOW_BOSSKEY
					}
					
					break;
			}
		}
		
		// Check for "Attrack mode"
		if(playOnceFinished) {
			menuAttrackMode();
		}
		
		SDL_Delay(50);
	}
	
	return true;
}

void menuShowNeedHelp() {
	
	// Lock Surface if needed
	if (SDL_MUSTLOCK(gScreen))
		if (SDL_LockSurface(gScreen) < 0) 
			return;	
	
	if (SDL_MUSTLOCK(menuneedhelpbg))
		if (SDL_LockSurface(menuneedhelpbg) < 0) 
			return;	
	
	
	Uint32 bigpitch = gScreen->pitch;
	Uint32 bigpitchBG = menuneedhelpbg->pitch;
	
	long fc_src_long = (long)menuneedhelpbg->pixels;
	long fc_dst_long = (long)gScreen->pixels;
	
	for(Uint32 j=0; j < SCR_HEIGHT; j++) {
		fc_src = (fastcopy *)fc_src_long;
		fc_dest = (fastcopy *)fc_dst_long;
		*fc_dest = *fc_src;
		fc_dst_long += bigpitch;
		fc_src_long += bigpitchBG;
	}
	
	
	// Unlock Surface if needed
	if (SDL_MUSTLOCK(menuneedhelpbg)) 
		SDL_UnlockSurface(menuneedhelpbg);
	if (SDL_MUSTLOCK(gScreen)) 
		SDL_UnlockSurface(gScreen);
	SDL_Flip(gScreen); /* Update whole screen */	
	
	bool joystickIdle = false;
	while(1) {
		SDL_Delay(50);
		
		// Handle Keyboard
		SDL_Event event;
		while (SDL_PollEvent(&event)) 
		{
			switch (event.type) 
			{
				case SDL_KEYUP:
					soundPlayFX(FX_MENU);
					return;
			}
		}
		Uint32 joymove = getJoystickMoves();
		if(joymove == JOYSTICK_NONE) {
			joystickIdle = true;
		} else if(joystickIdle) {
			soundPlayFX(FX_MENU);
			return;
		}
	}
}

void menuAttrackMode() {
	soundStopMusic();
	
	attracktModeRunning = true;
	attrackModeType = 0;
	while(attracktModeRunning) {
		switch(attrackModeType) {
			case 0:
				configSetAddOn("LostPixels");
				displayIntro();
				configResetAddOn();
				break;
			case 1:
				gamedata.player = &gamedata.players[0];
				gamedata.player->level = 1;
				gamedata.player->score = 0;
				gamedata.player->lives = 2;
				configSetAddOn("LostPixels");
				displayEngine("poscap_level_1.bin");
				configResetAddOn();
				break;
			case 2:
				gamedata.player = &gamedata.players[0];
				gamedata.player->level = 2;
				gamedata.player->score = 0;
				gamedata.player->lives = 2;
				configSetAddOn("LostPixels");
				displayEngine("poscap_level_2.bin");
				configResetAddOn();
				break;
			case 3:
				gamedata.player = &gamedata.players[0];
				gamedata.player->level = 3;
				gamedata.player->score = 0;
				gamedata.player->lives = 2;
				configSetAddOn("LostPixels");
				displayEngine("poscap_level_3.bin");
				configResetAddOn();
				break;
			case 4:
				gamedata.player = &gamedata.players[0];
				gamedata.player->level = 4;
				gamedata.player->score = 0;
				gamedata.player->lives = 2;
				configSetAddOn("LostPixels");
				displayEngine("poscap_level_4.bin");
				configResetAddOn();
				break;
			case 5:
				gamedata.player = &gamedata.players[0];
				gamedata.player->level = 5;
				gamedata.player->score = 0;
				gamedata.player->lives = 2;
				configSetAddOn("LostPixels");
				displayEngine("poscap_level_5.bin");
				configResetAddOn();
				break;
			case 6:
#ifdef HASOGGSUPPORT
				playogg("startanim_video.ogg",gScreen); /* play the standard ogg video instead of the BMF showVideo("startanim.bmf", true); */
#else // HASOGGSUPPORT	
				showVideo("startanim.bmf", true, true);
#endif // NO HASOGGSUPPORT
				break;
		}
		attrackModeType++;
		if(attrackModeType == 7) {
			attrackModeType = 0;
		}
	}
	soundStartMusic("ADDON/LostPixels/menuMusic.ogg", true);
}	
