// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "globals.h"
#include "playersprite.h"
#include "memops.h"
#include "errorhandler.h"
#include "engine.h"

SDL_Surface *SPRITE_Left;
SDL_Surface *SPRITE_Right;
SDL_Surface* sprite;
MOVE_TYPE lastmove = NO_MOVE;
Uint32 spritenum = 0;
Uint32 lastspritenum = 0;
Uint32 spritelasttick;
Uint32 spritetick;
Uint32 sprite_srcoffset = 0;
Uint32 playersprite_srcoffset;

void initPlayerSprite() {
	char fullfname[MAX_FNAME_LENGTH];
	sprintf(fullfname, "%s", configGetPath("sister_moveleft.bmp"));
	SDL_Surface* temp = IMG_Load(fullfname);
	if(!temp) {
		DIE(ERROR_IMAGE_READ, fullfname);
	}
	SPRITE_Left = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);

	sprintf(fullfname, "%s", configGetPath("sister_moveright.bmp"));
	temp = IMG_Load(fullfname);
	if(!temp) {
		DIE(ERROR_IMAGE_READ, fullfname);
	}
	SPRITE_Right = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	
	sprite = SPRITE_Left;
	
	spritetick = spritelasttick = SDL_GetTicks();
	
}


void deInitPlayerSprite() {
	SDL_FreeSurface(SPRITE_Left);
	SDL_FreeSurface(SPRITE_Right);
}

void blitSprite(Uint32 x, Uint32 y, SDL_Surface* src, Uint32 srcoffs) {
	if (SDL_MUSTLOCK(src))
		if (SDL_LockSurface(src) < 0) 
			return;
	
	Uint32 pitch_tmp = src->pitch / 4;
	Uint32 pitch_bg = gScreen->pitch / 4;	
	Uint32 i, j;
	Uint32 pixel;
	
	// Copy the sprite into the screen
	for(i=0; i < TILESIZE; i++) {
		for(j=0; j < TILESIZE; j++) {
			pixel = ((Uint32 *)src->pixels)[(j + TILESIZE * srcoffs) * pitch_tmp + i];
			
			if(pixel != 0x0000ff00) {
				((Uint32 *)gScreen->pixels)[(j + y) * pitch_bg + i + x] = pixel;
			}
			
		}
	}
	
	// Unlock Surfaces if needed
	if (SDL_MUSTLOCK(src)) 
		SDL_UnlockSurface(src);
}

void showSprite(Uint32 x, Uint32 y, MOVE_TYPE mtype) {
	spritetick = SDL_GetTicks();
	switch (mtype) {
		case MOVE_LEFT:
			sprite = SPRITE_Left;
			spritenum++;
			break;
		case MOVE_RIGHT:
			sprite = SPRITE_Right;
			spritenum++;
			break;
		case JUMP_LEFT:
			break;
		case JUMP_RIGHT:
			break;
		default:
			break;
	}
	if(lastmove != mtype || freeMovement) {
		spritenum = 0;
	} else {
		if(spritenum >= (Uint32)(sprite->h / TILESIZE)) {
			spritenum = 0;
		}
		if(lastspritenum != spritenum) {
			if(spritelasttick < spritetick) {
				spritelasttick = spritetick + SPRITE_SWITCHWAIT;
				lastspritenum = spritenum;
			} else {
				spritenum = lastspritenum;
			}
		}
	}
	playersprite_srcoffset = spritenum * TILESIZE;
	blitSprite(x, y, sprite, spritenum);	
	
	lastmove = mtype;
	
}

