#include "globals.h"
#include <string.h>
#include "triggers.h"
#include "fgobjects.h"
#include "errorhandler.h"
#include "engine.h"
#include "bl_lua.h"

TRIGGER gTriggers[1000];

void initTriggers() {
	memset(&gTriggers, 0, sizeof(gTriggers));
}

void deInitTriggers() {
	memset(&gTriggers, 0, sizeof(gTriggers));
}

void registerFGTrigger(Uint32 objid, bool isActive, char* funcname) {
	Uint32 i;
	bool found = false;

	if(objid >= fgobjcnt) {
		DIE(ERROR_INDEX_INVALID, "ObjectID does not exist");
	}

	for(i = 0; i < MAX_TRIGGERS; i++) {
		if(gTriggers[i].type == TRIGGER_TYPE_NONE) {
			found = true;
			break;
		}
	}
	if(!found) {
		DIE(ERROR_INDEX_INVALID, "Too many triggers");
	}

	gTriggers[i].type = TRIGGER_TYPE_FG;
	gTriggers[i].isActive = isActive;
	gTriggers[i].objid = objid;
	sprintf(gTriggers[i].funcname, "%s", funcname);
}
	
void setTriggerActive(Uint32 objid, bool isActive) {
	Uint32 i;
	bool found = false;

	for(i = 0; i < MAX_TRIGGERS; i++) {
		if(gTriggers[i].objid == objid) {
			gTriggers[i].isActive = isActive;
			found = true;
		}
	}
	if(!found) {
		DIE(ERROR_INDEX_INVALID, "ObjectID not found in Triggers");
	}
}


void handleTriggers() {
	Uint32 i;
	Sint32 playerx = (Sint32)spritex;
	Sint32 playery = (Sint32)spritey;

	for(i = 0; i < MAX_TRIGGERS; i++) {

		// Handle FG-Object triggers
		if(gTriggers[i].type == TRIGGER_TYPE_FG && gTriggers[i].isActive) {
			// Check if player CAN touch the object
			Uint32 objid = gTriggers[i].objid;
			if((playerx + TILESIZE) <= fgObjs[objid].x || (playery + TILESIZE) <= fgObjs[objid].y ||
			   (fgObjs[objid].x + FGOBJ_Surface[fgObjs[objid].gfxobj]->w) <= playerx ||
			   (fgObjs[objid].y + FGOBJ_Surface[fgObjs[objid].gfxobj]->h) <= playery) {
				// not overlapping
				continue;
			}
			if(checkPlayerFGTrueCollission(objid)) {
				blLuaCall(gTriggers[i].funcname, "i", objid);
			}
		}
	}

}
